/*!
 * VUEX-LOCALSTORAGE v0.1.2
 * (c) 2016 crossjs
 * Released under the MIT License.
 */
'use strict';

/**
 * createPersist
 * @param  {String} key             key
 * @param  {Object} [initialState]  初始值/默认值
 * @param  {Object} [config]        自定义 provider/serialize/deserialize/expires
 * @return {Object}                 get/set 方法
 */
function index (key) {
  var initialState = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var config = arguments[2];

  var _Object$assign = Object.assign({
    provider: localStorage,
    serialize: JSON.stringify,
    deserialize: JSON.parse,
    expires: 0 // never expires
  }, config);

  var provider = _Object$assign.provider;
  var serialize = _Object$assign.serialize;
  var deserialize = _Object$assign.deserialize;
  var expires = _Object$assign.expires;


  return {
    get: function get() {
      var state = void 0;

      try {
        var _deserialize = deserialize(provider.getItem(key));

        var value = _deserialize.value;
        var _expires = _deserialize.expires;

        if (_expires === 0 || _expires > new Date().getTime()) {
          state = deserialize(value);
        }
      } catch (e) {
        // console.log(e)
      }

      return Object.assign({}, initialState, state);
    },
    set: function set(value) {
      try {
        provider.setItem(key, serialize({
          value: serialize(value),
          expires: expires ? expires + new Date().getTime() : expires
        }));
      } catch (e) {
        // console.log(e)
      }
    }
  };
}

module.exports = index;